# -*- coding: utf-8 -*-

"""
Get ESRF data from 'results' folders in each process folder of each dataset.
"""

#--------------------------Imports-----------------------------
import os
from pathlib import Path
from pathlib import PurePosixPath
import sys
import collections
import stat
import datetime
import copy

# My modules
from modules.utils import verbose
from modules.utils import sftp
import config



#----------------------- Functions-------------

def test_path_proc_data(sftp_client, args):
    """
    Tests if path to processed data is correct
    :param sftp_client: sftp client connection with paramiko
    :param args: args from parser
    :return: beam, dates, acronym, and the list of paths of datasets to retrieve
    """
    if args.synchrotron == 'ESRF':
        beamlines = config.ESRF_BEAMLINES
        data_path = config.ESRF_PATH
    if args.synchrotron == 'ALBA':
        beamlines = config.ALBA_BEAMLINES
        data_path = config.ALBA_PATH

    mx = args.mx    # nom du mx (ex : mx2409)
    beam = args.beamline  # nom de la ligne
    dates = copy.deepcopy(args.dates)  # liste de dates sous forme YYYYMMDD
                                       # copie les valeurs sans assigner la variable à la même ref que args.dates
    acronym = args.protein  # nom de la prot
    group = args.group  # nom d'un éventuel niveau supplémentaire (group dans MxCube)
    paths_ok = []  # liste des paths des jeux à récupérer

    source_proc_data_common = PurePosixPath(data_path)

    # Cas ESRF :
    # path before /login/beam/acronym/beamline/date/...
    # Ex :/data/visitor/mx1896/id29/20170922/PROCESSED_DATA/erk2
    # => ajout au path de : mx et beam
    if not mx and args.synchrotron == 'ESRF':
        while True:
            mx = input("Proposal name (Ex: mx2409)? : ").lower()  # met le nom en minuscules
            if len(mx) >= 6:
                break
    if args.synchrotron == 'ESRF':
        source_proc_data_common = source_proc_data_common.joinpath(mx)
        try:
            # navigation sur la source
            sftp_client.chdir(str(source_proc_data_common))
        except FileNotFoundError as e:
            #print(str(e))
            print("\nProblem in the connection, check that the following parameter is correct: proposal name.")
        except PermissionError as e2:
            print("\nProblem in the connection, check that you are authorized to access the data.")
            sys.exit(0)
    if (not beam or beam not in beamlines) and args.synchrotron == 'ESRF':
        while True:
            beam = input(
                "Beamline name (" + ",".join(beamlines) + ")? : ").lower()  # met le nom en minuscules
            if beam in beamlines:
                break
    if args.synchrotron == 'ESRF':
        source_proc_data_common = source_proc_data_common.joinpath(beam)
        try:
            # navigation sur la source
            sftp_client.chdir(str(source_proc_data_common))
        except FileNotFoundError as e:
            #print(str(e))
            print("\nProblem in the connection, check that the following parameter is correct: beamline name.")
        except PermissionError as e2:
            print("\nProblem in the connection, check that you are authorized to access the data.")
            sys.exit(0)

    # cas de ESRF et ALBA => ajout au path de : date, PROCESSED_DATA, (group, acronyme)
    # Ex ALBA : /20230628/PROCESSED_DATA/jeu ou /20230628/PROCESSED_DATA/projet/jeu
    # vérification que le format de date est bon
    if dates:
        dates_ok = True
        for da in dates:
            if len(da) != 8:
                dates_ok = False
        if not dates_ok:
            print("\nProblem with the format of the list of dates.")
            print("Check that the list of dates is NOT in quotation marks.")
            sys.exit(0)
    else:
        while True:
            dates = input(
                "Date of the run? (format AAAAMMJJ) or list of dates for the same run "
                "(format AAAAMMJJ AAAAMMJJ) ? : ").split()
            dates_ok = True
            for da in dates:
                if len(da) != 8:
                    dates_ok = False
            if dates_ok:
                break
    # Ajouter un jour (cas d'un run de 24h, change de date après minuit)
    # pour chaque date (cas 2 dates non consécutives, comme pour une collecte un autre jour pour finir)
    new_dates = []  # nouvelle liste de dates
    dates.sort()    # pour les mettre dans l'ordre
    for date in dates:
        if date not in new_dates:   # gérer quand on donne 2 dates consécutives
            new_dates.append(date)
        date_temp = datetime.datetime.strptime(date, "%Y%m%d")
        new_date = date_temp + datetime.timedelta(days=1)
        new_dates.append(new_date.strftime("%Y%m%d"))
    dates_checked = []      # liste des dates avec des données
    for da in new_dates:  # parcourir la liste des dates pour un même run
        # vérifier le chemin vers "PROCESSED_DATA"
        try:
            source_proc_data = source_proc_data_common.joinpath(da, "PROCESSED_DATA")
            # navigation sur la source
            sftp_client.chdir(str(source_proc_data))
        except FileNotFoundError as e:
            #print(str(e))
            #print("\nProblem in the connection, check that the following parameter is correct: date = " + da + ".")
            pass
        except PermissionError as e2:
            print("\nProblem in the connection, check that you are authorized to access the data.")
            sys.exit(0)
        else:
            dates_checked.append(da)
    if not dates_checked:
        print("No date with data found ...")
        sys.exit(0)
    if not acronym:
        while True:
            acronym = input("Acronym? (if none, just press enter): ")
            break
    if not group:
        group = input("Is there a group (put in MxCube) as a parent folder for data? (if not, just press enter): ")
    date_checked_for_acro = []  # dates OK pour l'acronyme
    for da in dates_checked:  # parcourir la liste des dates pour un même run
        # gère le cas où un niveau de dossier supplémentaire (= groupe dans MxCube3)
        try:
            source_proc_data = source_proc_data_common.joinpath(da, "PROCESSED_DATA", group, acronym)
            print(source_proc_data)
            # navigation sur la source
            sftp_client.chdir(str(source_proc_data))
        except FileNotFoundError as e:
            #print(str(e))
            #print("\nProblem in the connection, check that the following parameter is correct: acronym = " + acronym +
            # (" and group = " + str(group) if group else "") + " for date = " + da + ".")
            pass
        except PermissionError as e2:
            print("\nProblem in the connection, check that you are authorized to access the data.")
            sys.exit(0)
        else:
            date_checked_for_acro.append(da)
            paths_ok.append(str(source_proc_data))
    if not date_checked_for_acro:
        print("No data found for these 'date, acronym and group' combination ...")
        sys.exit(0)
    return mx, beam, date_checked_for_acro, group, acronym, paths_ok



def list_proc_data(paths_ok, sftp_client, acronym, dates, synchrotron, proc_log):
    """
    Lists the data to be download and ask if the program should go on.
    :param paths_ok: list of paths of datasets to retrieve
    :param sftp_client: sftp_client connection with paramiko
    :param acronym: name of protein acronym
    :param dates: list of dates for runs
    :param synchrotron: which synchrotron
    :param proc_log: log file with path
    :return: jeux_proc = list of tuples (dataset, autoprocessing folder)
             recup_data = list of tuples (date, path, list of datasets OK, list of datasets without autoprocessing,
                            dictionary of the number of processes per dataset)
    """


    """List of the data to be retrieved"""
    recup_data = []  # liste de tuples (date, path, [jeux OK], [jeux sans autoprocessing], {dictionnaire du nombre de process par jeu})
    jeux_proc = []  # liste de tuples (jeu, dossier autoprocessing)
    paths_ok.sort()

    for path in paths_ok:
        jeux = []  # liste des jeux à copier pour chaque path
        jeux_ok = []  # liste des jeux avec autoprocessing
        jeux_without_proc = []  # liste des jeux sans autoprocessing
        sftp_client.chdir(path)
        """List of datasets"""
        # on vérifie que c'est un dossier (et que l'acronyme est dedans => pas forcément si en manuel)
        jeux = [j.filename for j in sftp_client.listdir_attr() if stat.S_ISDIR(j.st_mode)]    # and acronym in j.filename]
        jeux.sort()
        verbose.print_and_write("\nList of folders found in " + path + " (" + str(len(jeux)) + " folders) : ", proc_log)
        for j in jeux:
            verbose.print_and_write(j, proc_log)

        """List of autoprocessing folders for each dataset"""
        # ESRF
        # /data/visitor/mx2482/id30b/20230711/PROCESSED_DATA/LmNADK/LmNADK-L408-10min/autoprocessing_LmNADK-L408-10min_w1_run1_1
        # changement en sept 2023 :
        # /data/visitor/mx2482/id23eh1/20230920/PROCESSED_DATA/LmNADK/LmNADK-L407-CystN-coc-2/run_03_datacollection/autoprocessing_LmNADK-L407-CystN-coc-2_w1_run1_1
        # => changement pour un code plus robuste même si un peu plus de temps : fonction rglob pour sftp écrite pour chercher les dossiers "autoprocessing_"
        # ALBA :
        # /20230628/PROCESSED_DATA/B1X8/results => tous les mtz en vrac
        for jeu in jeux:
            sftp_client.chdir(jeu)
            if synchrotron == "ESRF":
                # faire un rglob pour si dossier intermédiaire // ATTENTION à faire sur sftp_client via paramiko !!!
                # => cf fonction paramiko_glob écrite pour ...
                autoproc_folders = sftp.paramiko_glob_folders(sftp_client.getcwd(), "autoprocessing_*", sftp_client)
                for f in autoproc_folders:
                    #print(jeu, f)  # TEST
                    subfolders = [item.filename for item in sftp_client.listdir_attr(f) if stat.S_ISDIR(item.st_mode)]
                    # tester que pas "autoprocessing_ref",ni "run_01_04_characterisation" (cf nouvelle organisation des données 2023)
                    # tester qu'il y a bien des sous-dossiers (sinon vide, cf nouvelle organisation des données 2023)
                    if "autoprocessing_ref" not in f and subfolders and "characterisation" not in f:
                        jeux_proc.append([jeu, f])
                        #print([jeu, f]) # TEST
            if synchrotron == "ALBA":
                # vérifier que le dossier 'results' existe (sinon caractérisation seulement)
                for l in sftp_client.listdir_iter():
                    if stat.S_ISDIR(l.st_mode) and l.filename == "results":
                        jeux_proc.append([jeu, 'results'])
            sftp_client.chdir(path)
        jeux_proc.sort(key=lambda t: (t[0], t[
            1].lower()))  # trier la liste de tuples selon le premier élément, puis le deuxième (casse indépendant => 'A' avant 'e')


        """calculation of the number of autoprocessing per dataset"""
        nbproc_par_jeu = collections.Counter(
            elem[0] for elem in jeux_proc)  # Counter => pas forcément dans l'ordre des élements
        sorted(nbproc_par_jeu.items())  # trier par item, cad par jeu
        verbose.print_and_write("\nNumber of 'processing' in each folder : ", proc_log)
        for j in jeux:
            verbose.print_and_write(j + ", nb proc : " + str(nbproc_par_jeu[j]), proc_log)
            if nbproc_par_jeu[j] != 0:  # count of a missing element is zero
                jeux_ok.append(j)
            else:
                jeux_without_proc.append(j)

        sorted(jeux_ok)
        sorted(jeux_without_proc)
        for date in dates:
            if date in path:
                recup_data.append((date, path, jeux_ok, jeux_without_proc, nbproc_par_jeu))
                break
    verbose.print_and_write(
        "\nList of datasets that will NOT be downloaded (" + str(sum(len(r[3]) for r in recup_data)) + ") :", proc_log)
    for date, path, j_OK, j_without_proc, nbp_par_j in recup_data:
        verbose.print_and_write("-for " + date + " (" + str(len(j_without_proc)) + ") : ", proc_log)
        for j in j_without_proc:
            verbose.print_and_write(j, proc_log)
    verbose.print_and_write("\nList of datasets that will be downloaded (" + str(sum(len(r[2]) for r in recup_data)) + ") :",
                    proc_log)
    for date, path, j_OK, j_without_proc, nbp_par_j in recup_data:
        verbose.print_and_write("-for " + date + " (" + str(len(j_OK)) + ") : ", proc_log)
        for j in j_OK:
            verbose.print_and_write(j, proc_log)
    verbose.print_and_write("\n", proc_log)

    if verbose.question_y_n_noaction("Shall we go on ?"):
        return jeux_proc, recup_data
    else:
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), proc_log)
        verbose.print_and_write("Program stopped by user ...", proc_log)
        sys.exit(0)


def download_results(sftp_client, data_dir, jeux_proc, recup_data, synchrotron, proc_log):
    """
    Download the processing subfolders (with only the files in the 'results' folders) for each autoprocessing
    :param sftp_client: sftp_client connection with paramiko
    :param data_dir: folder where data will be downloaded
    :param jeux_proc: list of tuples (dataset, autoprocessing folder path)
    :param recup_data: list of tuples (date, path, list of datasets OK, list of datasets without autoprocessing,
                            dictionary of the number of processes per dataset)
    :param synchrotron: which synchrotron
    :param proc_log: log file with path
    """
    jeux_deja = [] # liste des jeux déjà téléchargés
    jeux_copie = [] # liste des jeux copiés
    jeux_no_copie = [] # liste des jeux non copiés (pas de process)
    verbose.print_and_write("\nFiles downloaded:", proc_log)
    verbose.print_and_write("Data sets:", proc_log)
    for date, chemin, j_OK, j_without_proc, nbp_par_j in recup_data:
        sftp_client.chdir(chemin)
        for jeu in j_OK:
            # tester si le dossier jeu existe (cas où collecté sur 2 dates), et ne pas copier si dossier existe
            if not data_dir.joinpath(jeu).exists():
                jeux_copie.append(jeu)
                data_dir.joinpath(jeu).mkdir(exist_ok=True)
                sftp_client.chdir(jeu)
                verbose.print_and_write("\n- " + jeu, proc_log)
                nb = 0  # nb d'autoprocessing
                for (j, p) in jeux_proc:
                    if j == jeu and nb in range(nbp_par_j[jeu]):
                        sftp_client.chdir(p)
                        nb += 1
                        data_dir.joinpath(jeu, "PROC_" + str(nb)).mkdir(exist_ok=True)
                        verbose.print_and_write("\nNew directory: " + "PROC_" + str(nb), proc_log)

                        if synchrotron == "ESRF":
                            # lister les dossiers contenant les sous-dossiers 'results'
                            # TEMPO : attention, gérer si best_solution dans le nom du dossier contenant 'results', ne pas prendre
                            list_path_with_res = [path for (path, dos, f) in
                                                  sftp.sftp_walk(sftp_client, sftp_client.getcwd())
                                                  if ('results' in dos and 'best_solution' not in path)]
                            verbose.print_and_write("\nList of process directories: ", proc_log)
                            for path_result in list_path_with_res:
                                #                    sftp_client.chdir(path_result)
                                verbose.print_and_write("\n" + path_result, proc_log)
                                sftp_get_results(sftp_client, path_result, str(data_dir.joinpath(jeu, "PROC_" + str(nb))),
                                                 synchrotron, proc_log)
                        if synchrotron == "ALBA":
                            sftp_client.chdir(str(PurePosixPath(chemin).joinpath(jeu)))
                            path_result = sftp_client.getcwd() # /AAAAMMDD/PROCESSED_DATA/jeu/results => path jusqu'à jeu
                            verbose.print_and_write("\n" + path_result, proc_log)
                            sftp_get_results(sftp_client, path_result, str(data_dir.joinpath(jeu, "PROC_" + str(nb))),
                                                 synchrotron, proc_log)
                        sftp_client.chdir(str(PurePosixPath(chemin).joinpath(jeu)))
                sftp_client.chdir(chemin)
            else:
                jeux_deja.append(jeu)
                verbose.print_and_write("\n- " + jeu, proc_log)
                verbose.print_and_write("!!!!!!!!!!!!!!!!!!! The folder " + jeu + " already exists, "
                                                                                  "it has not been copied.", proc_log)
        for jeu in j_without_proc:
            jeux_no_copie.append(jeu)

    if jeux_no_copie:
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                    "List of datasets not copied (no process) (" + str(len(jeux_no_copie)) + "): "), proc_log)
        for jeu in jeux_no_copie:
            verbose.print_and_write(jeu, proc_log)

    # ajouter la liste des process 'sans mtz' pour info ??

    if jeux_copie:
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "List of processed datasets copied"
                                                " (" + str(len(jeux_copie)) + "): "), proc_log)
        for jeu in jeux_copie:
            verbose.print_and_write(jeu, proc_log)

    if jeux_deja:
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "Warning: " + str(len(jeux_deja)) +
                        " folder" + ("s" if len(jeux_deja) > 1 else "") + " not copied (already exists) ..."), proc_log)
        for jeu in jeux_deja:
            verbose.print_and_write(jeu, proc_log)



def sftp_get_results(sftp_client, path_res, proc_dir_local, synchrotron, log):
    """
    Download remotely result data in 'results' folder.
    Used in function 'download_results'
    Copy of the content of 'results' folder (which contains mtz) => only files, into the target folder in proc_dir_local
    :param sftp_client: sftp client connection with paramiko
    :param path_res: contains the results folder of the files to be copied
    :param proc_dir_local: folder where to copy processes by creating a folder (target folder) with the name of the process.
    :param synchrotron: which synchrotron
    :param log: log file with path
    """

    proc = Path(path_res).name  # nom du dossier sans le path

    # NEW 2023 : changement de l'emplacement du dossier 'results'
    # /data/visitor/mx2482/id23eh1/20230921/PROCESSED_DATA/HsNADK/HsNADK-apo-HsNADK7-A8-dioxane/run_04_datacollection/autoprocessing_HsNADK-apo-HsNADK7-A8-dioxane_w1_run1_1/XIA2_DIALS/nobackup/results
    # /data/visitor/mx2482/id23eh1/20230921/PROCESSED_DATA/HsNADK/HsNADK-apo-HsNADK7-C2-AcOEt/run_03_datacollection/autoprocessing_HsNADK-apo-HsNADK7-C2-AcOEt_w1_run1_1/grenades_parallelproc/nobackup/grenades_parallelproc/3_93.3_64.6_119.8_90_93.3_90/results
    # /data/visitor/mx2482/id23eh1/20230920/PROCESSED_DATA/HsNADK/HsNADK-apo-HsNADK7-H2-1_wellGOL-2s/run_03_datacollection/autoprocessing_HsNADK-apo-HsNADK7-H2-1_wellGOL-2s_w1_run1_1/grenades_fastproc/nobackup/grenades_fastproc/results
    # => changement de la récupération du nom du process car pas toujours au-dessus du dossier 'results', seulement quand = 'nobackup' car sinon écrase comme par ex pour les grenades-parallelproc!
    # => nom du dossier dans le path après 'autoprocessing_..."
    if proc == 'nobackup':
        # découper le path en liste de dossiers
        path_res_list = Path(path_res).parts
        #print(path_res_list)        # TEST
        # chercher la position dans la liste du dossier 'autoprocessing_..."
        indice_autoproc=0     # indice du dossier 'autoprocessing_..." dans la liste
        for i, dos in enumerate(path_res_list):
            if dos.startswith('autoprocessing_'):
                indice_autoproc = i
                break
        if indice_autoproc+1 < len(path_res_list):
            proc = path_res_list[indice_autoproc+1]
        else:   # dossier 'results' directement dans 'autoprocessing_...' => pour résultats MR
            proc = ''
    #print(proc)     # TEST

    # ATTENTION : cas où pas de dossier 'results' et les données sont là:
    # /data/visitor/mx2482/id23eh1/20230921/PROCESSED_DATA/HsNADK/HsNADK-apo-HsNADK6-G11/run_04_datacollection/autoprocessing_HsNADK-apo-HsNADK6-G11_w1_run1_1/grenades_fastproc/grenades_fastproc_anom
    # => chercher s'il y a un mtz et charger tout le dossier le contenant! ????????????????????????


#    print("Process directory name to create if mtz in results directory: " + proc)
    sftp_client.chdir(path_res)
    sftp_client.chdir('results')
    test_mtz = [f for f in sftp_client.listdir(sftp_client.getcwd()) if ".mtz" in f]
    if not proc:
        verbose.print_and_write("No process directory ...", log)
    elif test_mtz:   # s'il y a des fichiers mtz dans le dossier results
        #test = [f for f in sftp_client.listdir_attr(sftp_client.getcwd())]  # TEST
        #for f in test:
            #print(f)
            #print(f.st_mode)
        Path(proc_dir_local).joinpath(proc).mkdir(exist_ok=True)
        verbose.print_and_write("=> New directory created: " + proc, log)
        os.chdir(str(Path(proc_dir_local).joinpath(proc)))
        # gestion des liens symboliques dans le dossier 'results' comme dans le cas de parallel_proc (avec S_ISLNK)
        # files_to_copy = (f for f in sftp_client.listdir_attr(sftp_client.getcwd()) if (stat.S_ISREG(f.st_mode) or stat.S_ISLNK(f.st_mode)))
        # Pbm avec:
        # /data/visitor/mx2482/id30b/20230711/PROCESSED_DATA/NADK2/NADK2-cocADP-trempMnATP-5min/autoprocessing_NADK2-cocADP-trempMnATP-5min_w1_run1_1/grenades_codgas/best_solution_1_group_3/results
        #print("whereiam client? : ", sftp_client.getcwd())  # TEST
        #print("whereiam local? : ", os.getcwd())  # TEST
        files_to_copy = (f for f in sftp_client.listdir_attr(sftp_client.getcwd()) if (stat.S_ISREG(f.st_mode) or stat.S_ISLNK(f.st_mode)))
        for f in files_to_copy:
            try:
                print(f)    #TEST
                #print("ISREG: ", stat.S_ISREG(f.st_mode))
                #print("ISLNK: ", stat.S_ISLNK(f.st_mode))
                #verbose.print_and_write("File to copy: " + str(PurePosixPath(path_res).joinpath('results', f.filename)), log)
                #verbose.print_and_write("File to be copied: " + str(Path(proc_dir_local).joinpath(proc, f.filename)), log)
                sftp_client.get(str(PurePosixPath(path_res).joinpath('results', f.filename)), str(Path(proc_dir_local).joinpath(proc, f.filename)))
            except FileNotFoundError as e :
                verbose.print_and_write("Error: Problem of access to data ... ", log)
    else:
        verbose.print_and_write("No mtz files in this results directory ...", log)

    #sftp_client.chdir(path_res)
    #os.chdir(proc_dir_local)







