# -*- coding: utf-8 -*-

"""'launch' subcommand"""

# ----------------------- Imports --------------------------------
import os
import sys
from pathlib import Path
import datetime
from collections import Counter
import subprocess
import csv
import shutil

# My modules
import config
from modules.utils import verbose
from modules.utils import proteins
from modules.utils import software
from modules.subcommands import summary

#--------------------------Functions------------------------------
# Setup Phenix
if os.name == 'posix':
    env_phenix = software.shell_source(Path(config.PHENIX_PATH).joinpath("phenix_env.sh"))  # sourcer Phenix sous Linux
    shell_choice = False
elif os.name == 'nt':
    env_phenix = software.setup_bat(Path(config.PHENIX_PATH).joinpath("phenix_env.bat"))  # sourcer Phenix sous Windows
    shell_choice = True


def launch_run(data_dir, args, sg_ref):
    """
    'launch' subcommand
    :param data_dir: data directory
    :param args: args from parser
    :sg_ref: space group from reference pdb
    :return: log file
    """

    date = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")

    """Folder where we work"""
    work_dir = Path.cwd()
    """The existence of following path (created with 'prep' subcommand) has been checked in easypipe"""
    phenix_dir = work_dir.joinpath('easYPipe')
    prep_dir = phenix_dir.joinpath('1a_prep')
    mtz_treat_csv = prep_dir.joinpath('mtz_to_treat_ALL.csv')

    """Reference files checked in easypipe"""
    files = [f for f in Path(args.ref).resolve().iterdir() if f.is_file()]
    list_pdb = [f for f in files if f.resolve().suffix.lower() == '.pdb']
    list_fasta = [f for f in files if f.resolve().suffix.lower() in ['.fas', '.fa', '.fasta']]
    list_refcif = [f for f in files if f.resolve().suffix.lower() == '.cif']

    """Creation of launch directory"""
    launch_dir = phenix_dir.joinpath('2_launch')
    launch_dir.mkdir(exist_ok=True)

    """Create a log file with launch information"""
    launch_dir.joinpath('logs').mkdir(exist_ok=True)
    # nom du log avec nom du dossier avec ref
    nom_log = "launch_easypipe_" \
              + (sg_ref if (args.mode != 'allsg') else "allsg") \
              + ("_" + args.template if args.template else "")  \
              + "_" + args.mode.upper() + ("-LIG" if args.lig else "") + ("_" + str(args.nblig) if args.nblig > 1 and
                args.lig else "") \
              + ("_" + str(args.nblig) if args.nblig > 1 and args.lig else "") \
              + ("_CC" + str(args.cclig) if args.cclig != 0.7 and args.lig else "") + "_" \
              + ("whole" if args.whole else
                ("best" + str(args.best) if not args.autoproc else "autoPROC")) \
              + "_" + date + ("_simulate" if args.simulate else "") + ".log"

    launch_log = verbose.log_start(launch_dir, 'logs', nom_log, verbose.easypipe_logo)
    verbose.print_and_write("Mode: " + args.mode, launch_log)
    verbose.print_and_write(
        "Option: \n" + "\t- Best: " + str(args.best) + "\n\t- autoPROC: " + str(args.autoproc) + "\n\t- Whole: "
        + str(args.whole), launch_log)
    verbose.print_and_write("Ligand search: " + ("yes" if args.lig else "no"), launch_log)
    if args.cclig != 0.7 and args.lig:
        verbose.print_and_write(
            "Personalized minimum CC to consider a ligand placement correct: " + str(args.cclig), launch_log)
    if args.lig:
        verbose.print_and_write("Number of ligand copies to be searched: " + str(args.nblig),
                                      launch_log)
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
    verbose.print_and_write("Reference folder: " + args.ref, launch_log)
    verbose.print_and_write("Reference pdb files: " + ", ".join((lambda x: x.name)(x) for x in list_pdb),
                                  launch_log)  # écrire que les noms des fichiers
    verbose.print_and_write("Reference space group (from reference pdb files): " + sg_ref, launch_log)
    verbose.print_and_write("Reference fasta file: " + list_fasta[0].name,
                                  launch_log)  # écrire que le nom du fichier
    if list_refcif:
        verbose.print_and_write(
            "cif files for reference pdb files: " + ", ".join((lambda x: x.name)(x) for x in list_refcif), launch_log)

    """Check if reindexed csv file exists (in case)"""
    reindex_dir = phenix_dir.joinpath('1b_reindex_' + sg_ref)
    mtz_treat_csv_reindex = reindex_dir.joinpath(mtz_treat_csv.stem + "_reindexed.csv")
    mtz_treat_csv_OK = mtz_treat_csv
    if reindex_dir.exists() and args.mode != 'allsg':
        if mtz_treat_csv_reindex in [f for f in reindex_dir.iterdir() if f.is_file()]:
            mtz_treat_csv_OK = mtz_treat_csv_reindex
    else:
        mtz_treat_csv_OK = mtz_treat_csv    # cas 'allsg'

    verbose.print_and_write("File listing datasets to be treated: " + mtz_treat_csv_OK.name, launch_log)
    verbose.print_and_write("Creation of '2_launch' directory for 'launch' work", launch_log)

    """Get datasets names from 'mtz_to_treat_ALL.csv' or 'mtz_to_treat_ALL_reindexed.csv 'file"""
    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "Getting information from '"
                                               + mtz_treat_csv_OK.name + "' file"), launch_log)
    # liste des jeux
    jeux_name = []
    with open(mtz_treat_csv_OK, newline='') as csvfile:
        mtz_csv_reader = csv.DictReader(csvfile)
        for row in mtz_csv_reader:
            if row['dataset'] not in jeux_name:
                jeux_name.append(row['dataset'])
    verbose.print_and_write("Datasets to treat(" + str(len(jeux_name)) + "):", launch_log)
    for j in jeux_name:
        verbose.print_and_write(j, launch_log)

    """Looking for cif ligand files if ligand search"""
    if args.lig:
        jeux_cif = []  # liste de tuples jeu, cif du ligand
        jeux_no_cif = []  # liste des jeux sans cif
        # chercher cif dans dossier ligand pour chaque jeu
        verbose.print_and_write(
            verbose.cadre_text(config.motif, config.nb_motifs, "Listing cif files in datasets"),
            launch_log)
        for j in jeux_name:
            verbose.print_and_write("Dataset: " + j, launch_log)
            ligand_dir = phenix_dir.joinpath('0_processed_datasets', j, 'ligand')
            cif_in_lig = [f for f in ligand_dir.iterdir() if f.suffix.lower() == '.cif']
            if len(cif_in_lig) > 1:  # cas où plusieurs cif
                verbose.print_and_write("More than one cif file, only the first one will be considered ...",
                                              launch_log)
            if len(cif_in_lig) != 0:
                verbose.print_and_write("cif file for ligand search: " + cif_in_lig[0].name, launch_log)
                jeux_cif.append((j, cif_in_lig[0]))
            else:
                verbose.print_and_write("No cif file for ligand search in 'ligand' folder ...", launch_log)
                jeux_no_cif.append(j)
            verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                    launch_log)
        verbose.print_and_write("Datasets without cif file for ligand search(" + str(len(jeux_no_cif)) + "): ",
                                      launch_log)
        for j in jeux_no_cif:
            verbose.print_and_write(j, launch_log)

    """Names of csv that will be created, for testing if exists (if after simulate)"""
    # dossier 'sorted_mtz_csv_files' pour ranger tous les csv sauf ceux à modifier
    sorted_csv_dir = launch_dir.joinpath('sorted_mtz_csv_files')
    mtz_treat_csv_sorted = sorted_csv_dir.joinpath(mtz_treat_csv_OK.stem + "_sorted_"
                                                   + (sg_ref if (args.mode != 'allsg') else "allsg") + ".csv")
    mtz_treat_csv_option = sorted_csv_dir.joinpath(
        "mtz_to_treat_" + (sg_ref if (args.mode != 'allsg') else "allsg")
        +("_whole" if args.whole else
        ("_autoPROC" if args.autoproc else ("_best" + str(args.best))))
        + ".csv")
    launch_mtz_csv = mtz_treat_csv_option.parents[1].joinpath("launch_mtz_"
                            + (sg_ref if (args.mode != 'allsg') else "allsg")
                            + "_" + args.mode.upper() + ("-LIG" if args.lig else "")
                            + ("_" + str(args.nblig) if args.nblig > 1 and args.lig else "")
                            + ("_CC" + str(args.cclig) if args.cclig != 0.7 and args.lig else "")
                            + "_" + ("whole" if args.whole else ("best" + str(args.best) if not args.autoproc else "autoPROC"))
                            + ".csv")
    launch_mtz_csv_exist = False
    datasets_no_mtz_sg = sorted_csv_dir.joinpath("datasets_without_mtz_" + sg_ref + ".csv")  # ne sera pas créé si allsg

    if not launch_mtz_csv.exists() or args.simulate or args.subcommands == 'auto':
        # seulement si n'existe pas i.e. si pas déjà créé en simulation, ou si mode simulate / sauf en 'auto' où recréé

        """Creating a csv with sorted mtz to treat (right space group, decreasing completeness)"""
        # Rem : cas où échec de récup du sg OK, car pas conforme au sg demandé
        sorted_csv_dir.mkdir(exist_ok=True)
        verbose.print_and_write(
            verbose.cadre_text(config.motif, config.nb_motifs, "Creation of a csv file with mtz sorted"),
            launch_log)
        verbose.print_and_write("Creation of a csv file: " + mtz_treat_csv_sorted.name, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        with open(mtz_treat_csv_OK, newline='') as csvfile:
            mtz_csv_reader = csv.DictReader(csvfile)
            header_sort = mtz_csv_reader.fieldnames  # récupère le header du fichier 'mtz_treat_csv'
        with open(mtz_treat_csv_sorted, 'w', newline='') as csvfile_sort:
            csv_writer = csv.writer(csvfile_sort, quoting=csv.QUOTE_MINIMAL)
            csv_writer.writerow(header_sort)  # header
            verbose.print_and_write("Sorting mtz files to be treated:", launch_log)
            verbose.print_and_write(
                "\t- only mtz corresponding to space group of reference pdb file" if (args.mode != 'allsg')
                else "" + "\t- sorted by decreasing completeness",
                launch_log)
            verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                    launch_log)
            verbose.print_and_write(' / '.join(header_sort[1:]), launch_log)
            for j in jeux_name:
                verbose.print_and_write("\nDataset: " + j, launch_log)
                lines_jeu = []  # lignes du csv correspondant au jeu j
                with open(mtz_treat_csv_OK, newline='') as csvfile:
                    mtz_csv_reader = csv.DictReader(csvfile)  # réinitialisation de l'itérateur
                    for row in mtz_csv_reader:
                        # vérif sg OK ou lancement sans vérif sg i.e. option 'allsg'
                        if row['dataset'] == j and (row['space group'] == sg_ref or args.mode == 'allsg'):
                            lines_jeu.append(row)
                    # pour chaque jeu, trier par complétude décroissante :
                    # !! gérer le cas allsg où il peut y avoir des lignes sans info (completeness,reso, sg),
                    # qui dans les autres cas sont éliminées quand on trie par sg
                    lines_jeu_sorted = sorted(lines_jeu, key=lambda row: (float(row['completeness(%)'])
                                                                        if row['completeness(%)'] else 0), reverse=True)
                    verbose.print_and_write("Number of mtz files to be treated: " + str(len(lines_jeu_sorted)),
                                                  launch_log)
                    if len(lines_jeu_sorted) != 0:
                        verbose.print_and_write(
                            '\n'.join(' / '.join(list(row.values())[1:]) for row in lines_jeu_sorted), launch_log)
                    for row in lines_jeu_sorted:
                        csv_writer.writerow(list(row.values()))
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)

        """Writing datasets without mtz in a csv file"""
        nb_jeux_sans_mtz = 0
        jeux_nb_mtz = []  # tuple nom du jeu, nb de mtz à traiter
        for j in jeux_name:
            nb_mtz = 0
            with open(mtz_treat_csv_sorted, newline='') as csvfile:
                mtz_csv_reader = csv.DictReader(csvfile)
                for row in mtz_csv_reader:
                    if row['dataset'] == j:
                        nb_mtz += 1
            jeux_nb_mtz.append((j, nb_mtz))
        # jeux sans mtz
        nb_jeux_sans_mtz = Counter(elem[1] for elem in jeux_nb_mtz)[0]  # nb de fois où key = 0
        # Ex => Counter({0: 8, 2: 7, 6: 3, 4: 3, 8: 1})
        if nb_jeux_sans_mtz >= 1:
            verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                                   "Writing datasets without mtz in a csv file"),
                                          launch_log)
            verbose.print_and_write("Number of datasets without any mtz to be treated: " + str(nb_jeux_sans_mtz),
                                          launch_log)
            verbose.print_and_write("\nDatasets without any mtz to be treated: ", launch_log)
            for j, nb_mtz in jeux_nb_mtz:
                if nb_mtz == 0:
                    verbose.print_and_write(j, launch_log)
            # écrire les datasets sans mtz à traiter (mauvais sg) dans un fichier csv (dans dossier RESULTS)
            verbose.print_and_write(
                "Creation of a csv file: " + datasets_no_mtz_sg.name + " listing datasets without any mtz "
                                                                       "to treat according to reference space group.",
                launch_log)
            verbose.print_and_write("File created in folder: " + sorted_csv_dir.name, launch_log)
            with open(datasets_no_mtz_sg, 'w', newline='') as csvfile_no_mtz:
                csv_writer = csv.writer(csvfile_no_mtz, quoting=csv.QUOTE_MINIMAL)
                csv_writer.writerow(['dataset'])  # header
                for j, nb_mtz in jeux_nb_mtz:
                    if nb_mtz == 0:
                        csv_writer.writerow([j])

        """Checking options (best, autoproc) and listing mtz to be treated"""
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                               "Listing mtz to be treated according to option ..."),
                                      launch_log)
        verbose.print_and_write(
            "Creation of a csv file listing mtz according to option: " + mtz_treat_csv_option.name, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        with open(mtz_treat_csv_sorted, newline='') as csvfile:
            mtz_csv_reader = csv.DictReader(csvfile)
            header_sorted = mtz_csv_reader.fieldnames  # récupère le header du fichier 'mtz_to_treat_ALL_sorted'
        with open(mtz_treat_csv_option, 'w', newline='') as csvfile_option:
            csv_writer = csv.writer(csvfile_option, quoting=csv.QUOTE_MINIMAL)
            csv_writer.writerow(header_sorted)  # header
            verbose.print_and_write(
                "For each dataset, writing mtz files to be treated in this csv file, according to chosen option:",
                launch_log)
            if args.whole:
                verbose.print_and_write(
                    "=> The whole mtz will be treated, sorted by decreasing completeness: ", launch_log)
            elif args.autoproc:
                verbose.print_and_write("=> Only mtz from autoPROC will be treated, except if no autoPROC, "
                                              "then mtz with best completeness will be treated", launch_log)
            else:
                verbose.print_and_write(
                    "=> Number of mtz to treat sorted by decreasing completeness: " + str(args.best), launch_log)
            for j in jeux_name:
                best_count = 1
                lines_jeu = []  # lignes du csv correspondant au jeu j
                with open(mtz_treat_csv_sorted, newline='') as csvfile:
                    mtz_csv_reader = csv.DictReader(csvfile)
                    for row in mtz_csv_reader:
                        if row['dataset'] == j:
                            # option --autoproc
                            if args.autoproc:
                                if 'autoproc' in row['process name'].lower() and "_staraniso_" not in row['mtz file'].lower():
                                    lines_jeu.append(row)
                            elif best_count <= args.best or args.whole:
                                lines_jeu.append(row)
                                best_count += 1
                if args.autoproc and not lines_jeu:  # pas de mtz issu de autoPROC pour ce jeu, prend le premier mtz (best 1)
                    with open(mtz_treat_csv_sorted, newline='') as csvfile:
                        mtz_csv_reader = csv.DictReader(csvfile)
                        for row in mtz_csv_reader:
                            if row['dataset'] == j:
                                lines_jeu.append(row)
                                break
                for row in lines_jeu:
                    csv_writer.writerow(list(row.values()))



        """Listing mtz to be treated with information for running Phenix (mode, ligand ...)"""
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                    "Listing mtz to be treated with information for running Phenix"), launch_log)
        verbose.print_and_write("Creation of a csv file listing mtz with mode and ligand information: "
                                      + launch_mtz_csv.name, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        with open(launch_mtz_csv, 'w', newline='') as csvfile_mode:
            csv_writer = csv.writer(csvfile_mode, quoting=csv.QUOTE_MINIMAL)
            header_mode = header_sorted
            header_mode.extend(['mode', 'ligand search', 'CC', 'nb ligands', 'ligand', 'information', 'to treat'])  # header
            csv_writer.writerow(header_mode)
            verbose.print_and_write("For each dataset:"
                                          "\n\t- writing if ligand cif file is present for search"
                                          "\n\t- writing mode depending on mode asked, the presence (or not) of ligand cif file and data quality"
                                          "\n\t- writing information if mode is different from mode asked"
                                          "\n\t- writing result folder name", launch_log)
            with open(mtz_treat_csv_option, newline='') as csvfile:
                mtz_csv_reader = csv.DictReader(csvfile)
                for row in mtz_csv_reader:
                    # cellules du csv copiées
                    row_mtz = list(row.values())
                    # mode
                    information = ""    # à mettre dans col information
                    # !! gérer le cas allsg où il peut y avoir des lignes sans info (completeness,reso, sg),
                    # qui dans les autres cas sont éliminées quand on trie par sg
                    mtz_completeness = float(row['completeness(%)']) if row['completeness(%)'] else 0
                    mtz_resolution = float(row['resolution(A)']) if row['resolution(A)'] else 1000
                    if (mtz_completeness < config.compl_min or mtz_resolution > config.reso_min) and mtz_completeness != 0:
                        if mtz_completeness < config.compl_min and mtz_resolution > config.reso_min:
                            information = 'completeness < ' + str(config.compl_min) + '%' \
                                          + ' and resolution > ' + str(config.reso_min) + 'A'
                        elif mtz_completeness < config.compl_min:
                            information = 'completeness < ' + str(config.compl_min) + '%'
                        else:
                            information = 'resolution > ' + str(config.reso_min) + 'A'
                    if not information:
                        row_mtz.append(args.mode)
                    elif mtz_completeness == 0:  # cas pas d'information (arrive en allsg)
                        row_mtz.append('')
                    else:
                        # on ne lance pas en 'full' ou 'allsg' si complétude < config.compl_min ou réso > config.reso_min => en 'fast'
                        row_mtz.append('fast')
                    # ligand search, CC, nb ligands, ligand
                    if args.lig:
                        if row['dataset'] in jeux_no_cif or mtz_completeness < config.compl_min or mtz_resolution > config.reso_min:
                            # on ne lance pas de recherche de ligand si complétude < config.compl_min ou réso > config.reso_min => en 'fast' sauf allsg
                            row_mtz.extend(['no', '', '', ''])
                            if not information:     # si pas déjà un pbm qui fait qu'on est en fast
                                information = "no ligand cif file"
                        else:  # cas recherche ligand
                            row_mtz.extend(["yes", args.cclig, args.nblig])
                            for jeu, cif in jeux_cif:  # ajout du nom du ligand dans la colonne 'ligand', i.e. nom du cif sans extension
                                if row['dataset'] == jeu:
                                    row_mtz.append(cif.stem)
                    else:
                        row_mtz.extend(['no', '', '', ''])
                    # information
                    row_mtz.append(information)
                    # 'to treat'
                    # on ne lance pas si en allsg et complétude et/ou résolution trop mauvaises
                    if mtz_completeness == 0 or (information and args.mode == 'allsg'):
                        row_mtz.append('no')
                    else:
                        row_mtz.append('yes')
                    csv_writer.writerow(row_mtz)
    else:
        launch_mtz_csv_exist = True
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        verbose.print_and_write(
            "csv file listing mtz with mode and ligand information (" + launch_mtz_csv.name
            + ") already exists and will be used to launch Phenix ..."
            + "\nIf you want to update it (after adding ligand cif for example), run with simulation "
              "option first, before running again", launch_log)

    """Launching Phenix according to chosen mode, copy of csv files with information on mtz files to be treated"""
    if not args.simulate:  # si pas en mode simulation
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                        "Launching Phenix according to chosen mode and options ..."),
                                      launch_log)
        """Creation of 'RESULTS' directory"""
        res_sum_dir = phenix_dir.joinpath('RESULTS_' + (sg_ref if (args.mode != 'allsg') else "allsg") +
                                          ("_" + args.template if args.template else ""))
        verbose.print_and_write(
            "Creation of a folder " + res_sum_dir.name + " for summary of the results (if doesn't exists yet)",
            launch_log)
        res_sum_dir.mkdir(exist_ok=True)

        """Copy of csv files with information on mtz files from 1a_prep and 2_launch/sorted_mtz_csv_files/"""
        verbose.print_and_write("Copy of csv files from 'prep' and 'launch' steps:", launch_log)
        res_mtz_treated = res_sum_dir.joinpath('_mtz_treated')
        verbose.print_and_write(
            "Creation of a folder: " + res_mtz_treated.name + " in folder: " + res_sum_dir.name, launch_log)
        res_mtz_treated.mkdir(exist_ok=True)
        # csv all mtz
        verbose.print_and_write("Copy of csv file: " + mtz_treat_csv.name, launch_log)
        shutil.copyfile(str(mtz_treat_csv), str(res_mtz_treated.joinpath(mtz_treat_csv.name)))
        # csv all reindexed
        if reindex_dir.exists():
            verbose.print_and_write("Copy of reindexed csv file: " + mtz_treat_csv_OK.name, launch_log)
            shutil.copyfile(str(mtz_treat_csv_OK), str(res_mtz_treated.joinpath(mtz_treat_csv_OK.name)))
        # csv jeux sans mtz
        datasets_no_mtz = prep_dir.joinpath("datasets_without_mtz.csv")
        if datasets_no_mtz.exists():
            verbose.print_and_write("Copy of csv file: " + datasets_no_mtz.name, launch_log)
            shutil.copyfile(str(datasets_no_mtz), str(res_mtz_treated.joinpath(datasets_no_mtz.name)))
        # csv mtz pour ce sg
        verbose.print_and_write("Copy of csv file: " + mtz_treat_csv_sorted.name, launch_log)
        shutil.copyfile(str(mtz_treat_csv_sorted), str(res_mtz_treated.joinpath(mtz_treat_csv_sorted.name)))
        # csv jeu sans mtz pour ce sg
        if datasets_no_mtz_sg.exists():
            verbose.print_and_write("Copy of csv file: " + datasets_no_mtz_sg.name, launch_log)
            shutil.copyfile(str(datasets_no_mtz_sg), str(res_mtz_treated.joinpath(datasets_no_mtz_sg.name)))
        # csv mtz Phenix lancé
        if launch_mtz_csv.exists():
            launch_mtz_csv_counter = launch_mtz_csv.name
            # vérifier si existe, ajouter un compteur pour ne pas écraser
            launch_files = [f.name for f in res_mtz_treated.iterdir() if
                            f.suffix.lower() == '.csv' and launch_mtz_csv.stem in f.name]
            if launch_files: # il y a déjà des fichiers launch de copiés
                # lister les incréments existants (au cas où supression de certains par l'utilisateur)
                increm_nbs = [int(file_name.split(".csv")[0].rsplit("_",1)[1]) for file_name in launch_files]
                increm_nbs.sort()
                increm_nb_new = increm_nbs[-1] + 1 # nouveau n° pour l'incrémentation (le dernier +1)
            else:
                increm_nb_new = 1
            launch_mtz_csv_counter = launch_mtz_csv.stem + "_" + str(increm_nb_new) + ".csv"
            verbose.print_and_write("Copy of csv file: " + launch_mtz_csv_counter, launch_log)
            launch_mtz_csv_copy = res_mtz_treated.joinpath(launch_mtz_csv_counter)
            shutil.copyfile(str(launch_mtz_csv), str(launch_mtz_csv_copy))

        """Launching Phenix ..."""
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        with open(launch_mtz_csv_copy, newline='') as csvfile:
            mtz_csv_reader = csv.DictReader(csvfile)
            for row in mtz_csv_reader:
                verbose.print_and_write("Dataset: " + row['dataset'], launch_log)
                verbose.print_and_write("Process: " + row['mtz nb'] + "_" + row['process name'], launch_log)
                verbose.print_and_write("mtz to treat: " + row['mtz file'], launch_log)
                mtz_to_treat = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'data',
                                                   row['mtz nb'] + "_" + row['process name'], row['mtz file'])
                # arguments pour lancer Phenix
                args_phenix = ['phenix.ligand_pipeline', 'nproc=Auto', 'preserve_chain_id=True',
                               'refine.after_ligand.hydrogens=False', 'prune=False', 'keep_hetatms=True',
                               'refine.after_mr.update_waters=False', mtz_to_treat, list_fasta[0],
                               'prefix=' + row['dataset']]
                args_lig = []
                # cas cif avec pdb de ref
                if len(list_refcif) != 0:
                    for cif in list_refcif:
                        args_phenix.append('extra_cif=' + str(cif))
                # mode FAST
                args_fast = ['skip_xtriage=True', 'mr=False', 'quick_refine=True', 'build=False',
                             'reference_structure=' + str(list_pdb[0]),
                             str(list_pdb[0])]  # arguments à ajouter à args pour fast, un seul pdb de réf car pas de mr
                # mode full : arguments à ajouter
                # ?? ajouter structure de réf (plante quand plusieurs monomères) : 'reference_structure=' + list_pdb[0] pour que soient tous alignés
                # => a l'air de superposer par défaut à la réf utilisée pour le MR, probablement utile si on veux changer de ref / pdb utilisé pour le MR
                args_full = ['mr=Auto', 'autobuild.quick=True', 'quick_refine=True']
                # TODO 'phaser.sg_alt=None',  à remettre quand bug corrigé (signalé en sept 2020), évite que change le groupe d'espace !! A TESTER
                #  => ne marche tjs pas en oct 2022!!
                args_full.extend([str(pdb) for pdb in
                                  list_pdb])  # ajoute la liste des modèles pdb comme des str et pas comme une liste (différent append)
                # mode 'allsg' // pas de autobuild car long et pas de recherche de ligand en général donc pas nécessaire
                args_allsg = ['mr=True', 'quick_refine=False', 'build=False']  # arguments à ajouter à args pour 'allsg'
                args_allsg.extend([str(pdb) for pdb in
                                   list_pdb])  # ajoute la liste des modèles pdb comme des str et pas comme une liste (différent append)
                # option 'lig' // recherche de ligand
                if row['ligand search'] == 'yes':
                    verbose.print_and_write("Ligand to search: " + row['ligand'], launch_log)
                    for jeu, cif in jeux_cif:
                        if row['dataset'] == jeu:
                            args_lig = ['ligand=' + str(cif), 'ligand_copies=' + str(row['nb ligands']),
                                        'keep_input_restraints=True',
                                        ('min_ligand_cc_keep=' + str(row['CC']) if row['CC'] != 0.7 else "")]
                            # ligand copies = None => the same as for MR (i.e. nb of chains), mais ne marche pas, Auto non plus !
                # arguments pour lancer Phenix, selon le mode
                if row['mode'] == 'fast':
                    args_phenix.extend(args_fast)
                elif row['mode'] == 'full':
                    args_phenix.extend(args_full)
                elif row['mode'] == 'allsg':
                    args_phenix.extend(args_allsg)
                if row['ligand search'] == 'yes':
                    args_phenix.extend(args_lig)
                else:
                    # ne s'arrête pas si R/Rfree trop élevé (pas nécessaire car pas derecherche ligand $
                    # et comme ça on a le résultat dans le tableau plutôt q'une erreur
                    args_phenix.extend(['skip_ligand=True', 'stop_if_r_free_greater_than=None'])
                if row['mode'] != args.mode and row[
                    'to treat'].lower() == 'yes':  # cas pas de ligand ou pbm complétude ou résolution
                    verbose.print_and_write(row['information'], launch_log)
                # lancer Phenix dans le dossier résultat
                if row['to treat'].lower() != 'yes':  # si pas 'yes' dans 'to treat' alors ne lance pas
                    verbose.print_and_write(
                        "Phenix will not be launched ('no' for 'to treat' column in csv file)", launch_log)
                else:
                    verbose.print_and_write("Launched in mode: '" + row['mode'] + ("' with ligand search"
                                            if row['ligand search'] == 'yes' else "' without ligand search"), launch_log)

                    # nom du dossier résultat: RES_ + nom mtz + sg_ref + name? + nom mode + nom ligand si recherché + CC
                    res_dir = "RES_" + row['mtz nb'] + "_" + (sg_ref if (args.mode != 'allsg') else "allsg") + (
                        "_" + args.template if args.template else "") + "_" + row['mode'].upper() + \
                              ("-LIG_" + row['ligand'] + ( "_" + row['nb ligands'] if int(row['nb ligands']) > 1 else "") + \
                               ("_CC" + row['CC'] if row['CC'] != '0.7' else "") if row['ligand search'] == 'yes' else "")

                    # création du dossier RES_
                    verbose.print_and_write("Creation of directory: " + res_dir + " in 'result' directory",
                                                  launch_log)
                    result_dir = mtz_to_treat.parents[2].joinpath('results')
                    result_dir.mkdir(exist_ok=True)
                    result_dir.joinpath(res_dir).mkdir(exist_ok=True)
                    phenix_already = False
                    # vérifier si pipeline_1
                    if result_dir.joinpath(res_dir, 'pipeline_1').exists():
                        # refaire tourner si juste pipeline.eff et pipeline.log dans dossier pipeline_1 (i. e. failed)
                        if len(os.listdir(result_dir.joinpath(res_dir, 'pipeline_1'))) > 2:
                            phenix_already = True
                        else:
                            phenix_already = False
                            # Supprimer 'pipeline_1' sinon va créer un 'pipeline_2'
                            # Gérer le cas où 2 easypipe en même temps et l'un est en train d'écrire dans pipeline_1
                            # quand l'autre essaye de le supprimer ??? est-ce que ça marche ????
                            try:
                                shutil.rmtree(result_dir.joinpath(res_dir, 'pipeline_1'))
                            except OSError as e:
                                verbose.print_and_write(e, launch_log)
                                verbose.print_and_write("Warning: check that this process was really already"
                                                        " launched!!", launch_log)
                                phenix_already = True
                    if phenix_already:
                        verbose.print_and_write("Phenix was already launched ...", launch_log)
                    else:
                        verbose.print_and_write("Phenix launched ...", launch_log)
                        # aller dans le dossier résultat
                        os.chdir(str(result_dir.joinpath(res_dir)))
                        verbose.print_and_write("with command: " + ' '.join(str(a) for a in args_phenix),
                                                      launch_log)
                        try:
                            phenix = subprocess.Popen(args_phenix, stdout=subprocess.PIPE,
                                                      stderr=subprocess.PIPE, shell=shell_choice, env=env_phenix, encoding='utf-8')
                            phenix.communicate()
                        except IndexError as e:
                            print(e)
                verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                        launch_log)

        """Writing results summary csv file"""
        verbose.print_and_write(
            verbose.cadre_text(config.motif, config.nb_motifs, "Writing results summary csv file"),
            launch_log)

        # Results summary csv
        res_sum_name = "RESULTS_" + launch_mtz_csv.name.split('launch_mtz_')[1]
        # on utilise le même compteur que pour la copie du fichier launch dans mtz_treated, pour que corresponde
        res_sum_name_count = res_sum_name.split(".csv")[0] + "_" + str(increm_nb_new) + ".csv"
        results_summary_csv = res_sum_dir.joinpath(res_sum_name_count)
        verbose.print_and_write("Creation of a csv file for results: " + results_summary_csv.name, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        with open(results_summary_csv, 'w', newline='') as csvfile_results:
            csv_writer = csv.writer(csvfile_results, quoting=csv.QUOTE_MINIMAL)
            header_results = ['dataset', 'mtz nb', 'process name', 'mtz file', 'treated', 'mode', 'ligand search',
                              'ligand', 'ligand smiles', 'CC', 'nb ligands', 'result folder','SUCCESS',
                              'failing step', 'Resolution(A)', 'Completeness', 'Rwork', 'Rfree',
                              'space group (pdb)', 'unit cell (pdb)', 'Ligand', 'Nb of ligands found', 'LigandFit best CC',
                              'LigandFit individual CCs', 'Coot', 'Final R/Rf after manual refinement',
                              'Phenix first round after Coot']
            csv_writer.writerow(header_results)  # header
            # on reprend la liste des mtz du fichier csv launch_mtz_csv_copy
            # (au cas où un nouveau lancement avec un launch différent)
            with open(launch_mtz_csv_copy, newline='') as csvfile:
                mtz_csv_reader = csv.DictReader(csvfile)
                verbose.print_and_write(
                    "Searching for information in Phenix files and writing them in csv file:\n", launch_log)
                for row in mtz_csv_reader:
                    infos = []  # liste des info à écrire dans le csv
                    verbose.print_and_write("Dataset: " + row['dataset'], launch_log)
                    verbose.print_and_write("Process: " + row['mtz nb'] + "_" + row['process name'], launch_log)
                    # 'dataset','mtz nb', 'process name', 'mtz file'
                    infos.extend(list(row.values())[0:4])
                    # rechercher les infos
                    if row['to treat'].lower() == 'yes':  # cas où a été lancé
                        mtz_to_treat = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'data',
                                                           row['mtz nb'] + "_" + row['process name'], row['mtz file'])
                        # 'treated'
                        infos.append('yes')
                        # mode
                        infos.append(row['mode'])
                        # 'ligand search'
                        infos.append(row['ligand search'])
                        # si 'ligand search' = yes
                        if row['ligand search'] == 'yes':
                            # 'ligand'
                            infos.append(row['ligand'])
                            # 'ligand smiles'
                            ligand_dir = mtz_to_treat.parents[2].joinpath('ligand')
                            # si fichier smiles généré avec commande 'ligands',
                            # sera constitué du nom du ligand (issu du nom du cif, lui-même généré à partir du nom du ligand) + .smi
                            smi_file = ligand_dir.joinpath(row['ligand'] + '.smi')
                            if smi_file.is_file():
                                # lire le smiles dans le fichier
                                with open(smi_file, 'r', encoding="utf8", errors='ignore') as smi_f:
                                    smi = smi_f.readline().split()[0].strip()
                                    infos.append(smi)
                            else:
                                infos.append("")
                        else:
                            infos.extend(["", ""])
                        # CC
                        infos.append(row['CC'])
                        # 'nb ligands'
                        infos.append(row['nb ligands'])
                        # result folder
                        res_dir = "RES_" + row['mtz nb'] + "_" + (sg_ref if (args.mode != 'allsg') else "allsg") + (
                            "_" + args.template if args.template else "") + "_" + row['mode'].upper() + \
                            ("-LIG_" + row['ligand'] + ("_" + row['nb ligands'] if int(row['nb ligands']) > 1 else "") + \
                                   ("_CC" + row['CC'] if row['CC'] != '0.7' else "") if row['ligand search'] == 'yes' else "")
                        infos.append(res_dir)
                        # Si pas de fichier .out => Phenix n'est pas allé au bout de l'exécution
                        result_dir_mtz = mtz_to_treat.parents[2].joinpath('results', res_dir, 'pipeline_1')
                        out_file = list(result_dir_mtz.glob("*.out"))  # générator transformé en liste
                        if not out_file:  # i.e. échec
                            # 'SUCCESS'
                            infos.append("no")
                            verbose.print_and_write("Phenix FAILED ... cf pipeline.log", launch_log)
                            # récupérer le dernier titre du fichier et le copier dans un champ 'failing step' du tableur
                            # 'failing step'
                            pipeline_log = list(result_dir_mtz.glob("pipeline.log"))  # générator transformé en liste
                            if pipeline_log:
                                titre = ""      # dernier titre du fichier pipeline.log à récupérer
                                with open(pipeline_log[0], 'r', encoding="utf8", errors='ignore') as infile:
                                    lignes = infile.readlines()
                                    for li in reversed(lignes): # parcourir la liste à l'envers (générateur créé)
                                        if li.find(' ==') >= 0 and len(li) == 80: # dernier titre du fichier
                                            titre = li.split("== ")[1].split(" ==")[0]
                                            break
                                infos.append(titre)
                            else:
                                infos.append("no pipeline.log file, try to launch again")
                            # Resolution(A)
                            infos.append(row['resolution(A)'] + ' (mtz)')  # récup dans csv launch
                            # Completeness
                            infos.append(row['completeness(%)'] + ' (mtz)')  # récup dans csv launch
                            # Rwork and Rfree
                            infos.extend(["", ""])
                            # space group (pdb)
                            infos.append(row['space group'] + ' (mtz)')  # récup dans csv launch
                            # unit cell (pdb)
                            infos.append(row['unit cell'] + ' (mtz)')  # récup dans csv launch
                        else:
                            # 'SUCCESS', 'failing step'
                            infos.extend(["yes", ""])
                            # récupérer les infos dans le fichier .out
                            reso = ""
                            completeness = ""
                            Rwork = ""
                            Rfree = ""
                            with open(out_file[0], "r") as infile:
                                lignes = infile.readlines()
                                debut_summary = -1
                                for no, li in enumerate(lignes):
                                    if li.find("Summary") >= 0:
                                        debut_summary = no
                                for no, li in enumerate(lignes):
                                    if li.find("Resolution") >= 0 and no > debut_summary and not reso:
                                        reso = li.split("=")[1].strip()
                                    if li.find("Completeness") >= 0:
                                        completeness = li.split("=")[1].strip()
                                    if li.find("R-work") >= 0 and no > debut_summary:
                                        Rwork = li.split("=")[1].strip()
                                    if li.find("R-free") >= 0 and no > debut_summary and not Rfree:
                                        Rfree = li.split("=")[1].strip()
                            verbose.print_and_write("Resolution: " + reso + " A", launch_log)
                            verbose.print_and_write("Completeness: " + completeness, launch_log)
                            verbose.print_and_write("Rwork / Rfree: " + Rwork + " / " + Rfree, launch_log)
                            #  'Resolution', 'Completeness', 'Rwork', 'Rfree'
                            infos.extend([reso, completeness, Rwork, Rfree])
                            # Récupérer le space group dans le pdb
                            cell_group = []  # récupération du groupe d'espace (déjà cherché précédemment mais Phaser peut le changer!)
                            pdb_final = list(result_dir_mtz.glob("*_final.pdb"))  # générator transformé en liste
                            if pdb_final:
                                cell_group = proteins.sg_from_pdb(pdb_final[0])
                                verbose.print_and_write("Space group (pdb file): " + cell_group, launch_log)
                                unit_cell = proteins.unitcell_from_pdb(pdb_final[0])
                                verbose.print_and_write("Unit cell (pdb file): " + unit_cell, launch_log)
                            # 'space group (pdb)'
                            if cell_group:
                                infos.append(cell_group)
                            else:
                                infos.append("space group not in official list or not found, see pdb file")
                            # 'unit cell (pdb)'
                            infos.append(unit_cell)
                            # Cas recherche de ligand !!!!!!!!!!!!!!!
                            # fichier summary.txt, + fichier FAILED => ligand pas trouvé, fichier FINISHED => ligand trouvé
                            # CC => dans le fichier pipeline.log à "Overall CC  =" (= best CC si un seul ligand cherché)
                            # si plusieurs ligands placés: (copy  1  =, (copy  2  = ...
                            res_files_names = list(
                                f.name for f in result_dir_mtz.glob("*"))  # générator transformé en liste
                            # cas recherche de ligand
                            # 'Ligand', 'Nb of ligands found', 'LigandFit best CC', 'LigandFit individual CC'
                            if 'FAILED' in res_files_names or 'FINISHED' in res_files_names:
                                lig_search = 'Not found'
                                nblig_found = '0/' + str(args.nblig)
                                verbose.print_and_write("Ligand searched (LigandFit)", launch_log)
                                bestCC = ""
                                pipeline_log = list(result_dir_mtz.glob("pipeline.log"))  # générator transformé en liste
                                if int(row['nb ligands']) == 1 and pipeline_log:
                                    with open(pipeline_log[0], 'r', encoding="utf8", errors='ignore') as infile:
                                        lignes = infile.readlines()
                                        for li in lignes:
                                            if li.find('Overall CC') >= 0:
                                                bestCC = li.split("=")[1].strip()
                                # récupérer les CC des ligands placés
                                CCs = []  # liste des CC individuels
                                if int(row['nb ligands']) > 1 and pipeline_log:
                                    with open(pipeline_log[0], "r", encoding="utf8", errors='ignore') as infile:
                                        lignes = infile.readlines()
                                        for li in lignes:
                                            if li.find('(copy  ') >= 0:
                                                CCs.append(li.split("=")[1].split(")")[0].strip())
                                    # trouver le bestCC
                                    bestCC = max(CCs)
                                if 'FAILED' in res_files_names:  # ligand pas trouvé
                                    verbose.print_and_write("Ligand not found ...", launch_log)
                                elif 'FINISHED' in res_files_names:  # ligand trouvé
                                    finished = list(f for f in result_dir_mtz.glob("FINISHED"))[0]  # générator transformé en liste
                                    with open(finished, 'r') as finishedf:
                                        nblig_found = finishedf.readline()
                                    lig_search = 'FOUND'
                                    verbose.print_and_write("Ligand FOUND !!!", launch_log)
                                verbose.print_and_write("Best CC: " + bestCC, launch_log)
                                verbose.print_and_write("CCs: " + ','.join(CCs), launch_log)
                                # 'Ligand', 'Nb of ligands found', 'LigandFit best CC', 'LigandFit individual CC'
                                infos.extend([lig_search, nblig_found, bestCC, ','.join(CCs)])
                            else:  # pas de ligand cherché
                                infos.extend(['No search', '', '', ''])
                            # Phenix first round
                            if os.name == 'posix':
                                # TODO : ajouter le cif du dossier ligand dans tous les cas ??
                                copyname = row['dataset'] + "_" + row['mtz nb'] \
                                               + (('_CC' + str(row['CC']) + '_nblig' + str(row['nb ligands']))
                                                  if row['ligand search'] == 'yes' else "") \
                                               + "_" + row['mode'] + \
                                               ("-lig" if row['ligand search'] == 'yes' else "")
                                mtzcopy_name = copyname + ".mtz"
                                pdbcopy_name = copyname + ".pdb"
                                cifcopy_name = copyname + ".cif"
                                phenix_first_round = " ".join(['phenix.refine', mtzcopy_name, pdbcopy_name.split('.pdb')[0]
                                                               + '-coot-0.pdb',
                                                               cifcopy_name if ('FINISHED' in res_files_names) else '',
                                                               'ordered_solvent=True',
                                                               'strategy=individual_sites+individual_adp',
                                                               'input.xray_data.labels="F-obs(+)"',
                                                               'prefix=' + row['dataset']
                                                               + '_' + row['mtz nb'] + "_" + row['mode'] + '_Phenix', 'serial=1'])
                                infos.extend(['', '', phenix_first_round])
                    else:
                        verbose.print_and_write("Phenix not launched ...", launch_log)
                        # 'treated'
                        infos.append('no')
                    # écrire les infos
                    csv_writer.writerow(infos)
                    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                            launch_log)


        """Copy of pdb and mtz result files, ligand cif and pdb, pdb of ligand(s) placed by LigandFit (all CCs) 
        and data files"""
        # Copier les pdb, mtz + cif et pdb du ligand pour chaque mtz
        verbose.print_and_write(
            verbose.cadre_text(config.motif, config.nb_motifs, "Copy of pdb and mtz result files"),
            launch_log)
        verbose.print_and_write("Creation of datasets folders in " + res_sum_dir.name + " folder and copy of "
                                                                                              "results files",
                                      launch_log)
        with open(results_summary_csv, newline='') as csvfile:
            mtz_csv_reader = csv.DictReader(csvfile)
            for row in mtz_csv_reader:
                if row['SUCCESS'] == 'yes':  # seulement si traitement OK
                    verbose.print_and_write("\nDataset: " + row['dataset'], launch_log)
                    # créer dossier dataset
                    datacopy_dir = res_sum_dir.joinpath(row['dataset'])
                    datacopy_dir.mkdir(exist_ok=True)
                    # dossier où sont les fichiers à copier
                    result_dir_mtz = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'results',
                                                         row['result folder'])
                    copyname = row['dataset'] + "_" + row['mtz nb'] \
                                               + (('_CC' + str(row['CC']) + '_nblig' + str(row['nb ligands']))
                                                  if row['ligand search'] == 'yes' else "") \
                                               + "_" + row['mode'] + \
                                               ("-lig" if row['ligand search'] == 'yes' else "")
                    # pdb final
                    pdb_final = list(result_dir_mtz.rglob("*_final.pdb"))  # générator transformé en liste
                    if pdb_final:
                        verbose.print_and_write("Copy of pdb file: " + pdb_final[0].name, launch_log)
                        pdbcopy_name = copyname + ".pdb"
                        verbose.print_and_write("to: " + datacopy_dir.name + "/" + pdbcopy_name, launch_log)
                        shutil.copyfile(str(pdb_final[0]), str(datacopy_dir.joinpath(pdbcopy_name)))
                    # mtz final
                    mtz_final = list(result_dir_mtz.rglob("*_final.mtz"))  # générator transformé en liste
                    if mtz_final:
                        verbose.print_and_write("Copy of mtz file: " + mtz_final[0].name, launch_log)
                        mtzcopy_name = copyname + ".mtz"
                        verbose.print_and_write("to: " + datacopy_dir.name + "/" + mtzcopy_name, launch_log)
                        shutil.copyfile(str(mtz_final[0]), str(datacopy_dir.joinpath(mtzcopy_name)))
                    """
                    # mtz data
                    mtz_data = list(result_dir_mtz.rglob("*_data.mtz"))  # générator transformé en liste
                    if mtz_data:
                        verbose.print_and_write("Copy of mtz data file: " + mtz_data[0].name, launch_log)
                        mtzdatacopy_name = mtz_data[0].stem.split('data')[0] + row['mtz nb'] + "_" + row[
                            'mode'] + "_data.mtz"
                        verbose.print_and_write("to: " + datacopy_dir.name + "/" + mtzdatacopy_name, launch_log)
                        shutil.copyfile(str(mtz_final[0]), str(datacopy_dir.joinpath(mtzdatacopy_name)))
                    """
                    # cif final (cas recherche ligand aboutie)
                    cif_final = list(result_dir_mtz.rglob("*_final.cif"))  # générator transformé en liste
                    if cif_final:
                        verbose.print_and_write("Copy of found ligand cif file: " + cif_final[0].name, launch_log)
                        cifcopy_name = copyname + ".cif"
                        verbose.print_and_write("to: " + datacopy_dir.name + "/" + cifcopy_name, launch_log)
                        shutil.copyfile(str(cif_final[0]), str(datacopy_dir.joinpath(cifcopy_name)))
                    # copy of mtz and log data
                    # dossier data avec les mtz et log, vérif bon mtz_nb
                    data_mtz_log = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'data',
                                                       row['mtz nb'] + '_' + row['process name'])
                    # création d'un dossier data + mtz_nb
                    data_mtz_log_copydir = datacopy_dir.joinpath('data_' + row['mtz nb'])
                    data_mtz_log_copydir.mkdir(exist_ok=True)
                    # copie de tout sauf fichier avec verbose dans le nom
                    mtz_log_to_copy = (f for f in data_mtz_log.iterdir() if f.is_file() and 'verbose' not in f.name)
                    verbose.print_and_write("Copy of corresponding data files in "
                                            + data_mtz_log_copydir.name, launch_log)
                    for f in mtz_log_to_copy:
                        if not data_mtz_log_copydir.joinpath(f.name).exists():
                            verbose.print_and_write("\t" + f.name, launch_log)
                            shutil.copyfile(str(f), str(data_mtz_log_copydir.joinpath(f.name)))
                        else:
                            verbose.print_and_write("\t" + f.name + "already copied", launch_log)
                    # copier pdb et cif ligand dans dossier 'ligand', si pas de pdb cf dossier elBOW
                    ligand_dir = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'ligand')
                    # on vérifie qu'il y a un cif ou un pdb dans le dossier ligand
                    ligand_dir_files = [f for f in ligand_dir.iterdir() if f.name.endswith('.cif')
                                        or f.name.endswith('.pdb')]
                    if ligand_dir.exists() and ligand_dir_files:
                        # création d'un dossier ligand
                        ligandcopy_dir = datacopy_dir.joinpath('ligand')
                        ligandcopy_dir.mkdir(exist_ok=True)
                        lig_cif = list(ligand_dir.glob("*.cif"))
                        lig_pdb = list(ligand_dir.glob("*.pdb"))
                        if lig_cif:
                            verbose.print_and_write("Copy of ligand cif file: " + lig_cif[0].name, launch_log)
                            shutil.copyfile(str(lig_cif[0]), str(ligandcopy_dir.joinpath(lig_cif[0].name)))
                        if lig_pdb:
                            verbose.print_and_write("Copy of ligand pdb file: " + lig_pdb[0].name, launch_log)
                            shutil.copyfile(str(lig_pdb[0]), str(ligandcopy_dir.joinpath(lig_pdb[0].name)))
                        else:  # chercher dans dossier /pipeline_1/eLBOW_0/
                            elbow_dir = result_dir_mtz.joinpath('pipeline_1', 'eLBOW_0')
                            if elbow_dir.exists():
                                verbose.print_and_write("Copy of ligand cif and pdb files from "
                                                              + '/'.join(elbow_dir.parts[-2:]) + " ...", launch_log)
                                lig_cif_elbow = list(elbow_dir.glob("*.cif"))
                                lig_pdb_elbow = list(elbow_dir.glob("*.pdb"))
                                if lig_cif_elbow:
                                    verbose.print_and_write("Copy of ligand cif file: " + lig_cif_elbow[0].name,
                                                                  launch_log)
                                    shutil.copyfile(str(lig_cif_elbow[0]), str(ligandcopy_dir.
                                                                               joinpath(lig_cif_elbow[0].name)))
                                if lig_pdb_elbow:
                                    verbose.print_and_write("Copy of ligand pdb file: " + lig_pdb_elbow[0].name,
                                                                  launch_log)
                                    shutil.copyfile(str(lig_pdb_elbow[0]), str(ligandcopy_dir.
                                                                               joinpath(lig_pdb_elbow[0].name)))
                        # copy of pdb of ligand(s) placed by LigandFit (all CCs), dans dossier 'ligand' en ajoutant mtznb
                        ligandFit_pdb = list(result_dir_mtz.rglob("ligand_fit_*.pdb"))  # générator transformé en liste
                        if ligandFit_pdb:
                            verbose.print_and_write("Copy of ligand pdb files from LigandFit: ", launch_log)
                            for ligfpdb in ligandFit_pdb:
                                verbose.print_and_write(ligfpdb.name, launch_log)
                                ligfpdb_copyname = 'ligand_fit' + copyname.split(row['dataset'])[1] \
                                                   + ligfpdb.name.split('ligand_fit')[1]
                                verbose.print_and_write("to: " + ligfpdb_copyname, launch_log)
                                shutil.copyfile(str(ligfpdb), str(ligandcopy_dir.joinpath(ligfpdb_copyname)))

        """'summary' command run automatically"""
        os.chdir(work_dir)
        summary.summary_run(data_dir, [res_sum_dir.name], args)

    else:  # mode simulate
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        verbose.print_and_write("Simulation mode, Phenix not launched ...\n", launch_log)
    #        verbose.print_and_write("Now you have the possibility to modify the csv file:" + launch_mtz_csv.name
    #                        + "\nin 'launch' folder to choose not to launch certain treatments "
    #                        "(just change 'yes' for 'no' in the cell for 'to treat' column), "
    #                        "before restarting the same command without simulation option.", launch_log)

    """Summary"""
    # TODO : ajouter bilan success/échec (nombre) ? ou compter le nb de dossier jeu avec res copiés, et déduire qui n'a pas de res
    verbose.print_and_write(verbose.chaine_xfois(config.motif, config.nb_motifs), launch_log)
    verbose.print_and_write(verbose.easypipe_logo, launch_log)
    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "SUMMARY"),
                            launch_log)
    verbose.print_and_write("Mode: " + args.mode, launch_log)
    verbose.print_and_write(
        "Option: \n" + "\t- Best: " + str(args.best) + "\n\t- autoPROC: " + str(args.autoproc) + "\n\t- Whole: "
        + str(args.whole), launch_log)
    if args.mode != 'allsg':
        verbose.print_and_write("Reference space group: " + sg_ref, launch_log)
    verbose.print_and_write("Working directory: " + launch_dir.name, launch_log)
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
    # si recherche ligand et jeu sans cif, les lister
    if args.lig:
        if args.cclig != 0.7:
            verbose.print_and_write(
                "Personalized minimum CC to consider a ligand placement correct: " + str(args.cclig),
                launch_log)
        verbose.print_and_write("Number of ligand copies to be searched: " + str(args.nblig), launch_log)
        verbose.print_and_write("Ligand searched for datasets with cif file in 'dataset'/ligand folder.",
                                      launch_log)
        if jeux_no_cif:
            verbose.print_and_write(
                "Datasets without cif file for ligand search(" + str(len(jeux_no_cif)) + "): ", launch_log)
            for j in jeux_no_cif:
                verbose.print_and_write(j, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)

    if not launch_mtz_csv_exist or args.simulate:
        # csv file avec mtz trié selon complétude et sg
        verbose.print_and_write(mtz_treat_csv_sorted.name + ": csv file with mtz sorted, \nin folder: "
                                      + '/'.join(mtz_treat_csv_sorted.parts[-3:-1]), launch_log)
        verbose.print_and_write("mtz files to be treated sorted:", launch_log)
        verbose.print_and_write(
            "\t- only mtz corresponding to space group of reference pdb file" if (args.mode != 'allsg') else ""
            + "\t- sorted by decreasing completeness",
            launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        # s'il y a des jeux sans mtz candidat au traitement, listés dans un csv
        if nb_jeux_sans_mtz >= 1:
            verbose.print_and_write("Datasets without mtz written in a csv file: " + datasets_no_mtz_sg.name,
                                          launch_log)
            verbose.print_and_write("Number of datasets without any mtz to be treated: " + str(nb_jeux_sans_mtz),
                                          launch_log)
            verbose.print_and_write("\nDatasets without any mtz to be treated: ", launch_log)
            for j, nb_mtz in jeux_nb_mtz:
                if nb_mtz == 0:
                    verbose.print_and_write(j, launch_log)
            verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                    launch_log)
        # csv avec les options
        verbose.print_and_write(
            mtz_treat_csv_option.name + ": csv file listing mtz files to be treated according to chosen "
                                        "option \nin folder: " + '/'.join(mtz_treat_csv_option.parts[-3:-1]),
            launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        # csv avec infos pour lancement
        verbose.print_and_write(
            "Creation of a csv file listing mtz with mode and ligand information for running "
            "phenix.ligand_pipeline: " + launch_mtz_csv.name, launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
    else:
        if not args.simulate:
            verbose.print_and_write(
                "csv file listing mtz with mode and ligand information (" + launch_mtz_csv.name
                + ") already exists and was used to launch Phenix ...", launch_log)
            verbose.print_and_write(
                "\nIf something has changed, like adding ligands for example, launch again with "
                "simulate option to re-generate the csv file, then launch again.", launch_log)
            verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                    launch_log)
    if not args.simulate:  # si pas en mode simulation
        # phenix
        verbose.print_and_write("phenix.ligand_pipeline was launched according to chosen mode and option ...",
                                      launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        # csv résultat
        verbose.print_and_write("Results summary csv file: " + results_summary_csv.name + " \nwritten in: "
                                      + '/'.join(results_summary_csv.parts[-3:-1]), launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        # Copier les pdb, mtz + cif et pdb du ligand pour chaque mtz
        verbose.print_and_write("Copy of pdb and mtz result files in datasets folders in " + res_sum_dir.name,
                                      launch_log)
        #        verbose.print_and_write("Copy of data mtz file", launch_log)
        if args.lig:
            verbose.print_and_write("Copy of phenix cif file if ligand found", launch_log)
        verbose.print_and_write("Copy of ligand folder, if exists, for each dataset", launch_log)
        verbose.print_and_write("Copy of pdb of ligand(s) placed by LigandFit (all CC)", launch_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), launch_log)
        verbose.print_and_write(
            "Copy of some csv files from 'prep' and 'launch' steps to " + res_mtz_treated.name, launch_log)
    else:  # mode simulate
        verbose.print_and_write("Simulation mode, Phenix not launched ...\n", launch_log)
        verbose.print_and_write("Now you have the possibility to modify the csv file: " + launch_mtz_csv.name
                                      + "\nin '2_launch' folder to choose not to launch certain treatments "
                                        "(just change 'yes' for 'no' in the cell for 'to treat' column), "
                                        "before restarting the same command without simulation option.", launch_log)
    os.chdir(work_dir)
    return launch_log

