# -*- coding: utf-8 -*-

"""'summary' subcommand """

# ----------------------- Imports --------------------------------
import os
from pathlib import Path
import datetime
import csv
from collections import OrderedDict

# My modules
import config
from modules.utils import verbose

#--------------------------Functions------------------------------
def summary_run(data_dir, res_dirs, args):
    """
    'summary' subcommand
    :param data_dir: data directory
    :param res_dirs: list of results directories names (in 'easYPipe' folder)
    :param args: args from parser
    :return: log file
    """

    date = datetime.datetime.now().strftime("%Y-%m-%d_%H%M%S")

    """Folder where we work"""
    work_dir = Path.cwd()
    """The existence of following path (created with 'prep' subcommand) has been checked in easypipe"""
    phenix_dir = work_dir.joinpath('easYPipe')

    """Create a log file"""
    nom_log = "summary_RESULTS_easYPipe_" + date + ".log"
    summary_log = verbose.log_start(phenix_dir, 'logs-summary-RESULTS', nom_log, verbose.easypipe_logo)

    """List of datasets without successfull treatment"""
    datasets_all = []
    datasets_treated = []    # tous les jeux traités
    datasets_success = []


    """Create a summary file for each RESULTS dir"""
    # liste les fichiers résultats dans le(s) dossier(s) RESULTS_...

    # si plusieurs RESULTS dir
    unsuccessfull_all = False  # test pour savoir s'il y a des traitements 'SUCCESS = no' sur l'ensemble
    rows_summary_files = []  # lignes des différents csv summary si plusieurs RESULTS dir

    for dir in res_dirs:
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                   "Result directory: " + Path(dir).name), summary_log)
        res_files = [f for f in phenix_dir.joinpath(dir).iterdir() if f.is_file() and f.name.startswith('RESULTS')
                     and f.suffix.lower() == '.csv']    # ne prend pas les .xlsx si on en a créé
        verbose.print_and_write("List of RESULT csv files:", summary_log)
        unsuccessfull = False  # test pour savoir s'il y a des traitements SUCCESS = no

        if res_files: # gère si pas encore de fichier RESULTS_... (cas plusieurs dossiers RESULTS et un avec calculs pas finis)
            header = []
            for file in res_files:
                verbose.print_and_write(file.name, summary_log)
            verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), summary_log)

            # récupère le header
            if not header:
                with open(res_files[0], newline='') as csvfile:
                    mtz_csv_reader = csv.DictReader(csvfile)
                    header = mtz_csv_reader.fieldnames  # récupère le header

            rows_all_files = []  # liste de liste de cellules (ligne du csv)
            for file in res_files:
                #verbose.print_and_write("File: " + file.name, summary_log)
                with open(file, newline='') as csvfile:
                    mtz_csv_reader = csv.DictReader(csvfile)
                    for row in mtz_csv_reader:
                        if row['dataset'] not in datasets_all:
                            datasets_all.append(row['dataset'])
                        # vérif si treated = 'yes'
                        if row['treated'] == 'yes':
                            rows_all_files.append(row)
                            if row['dataset'] not in datasets_treated:
                               datasets_treated.append(row['dataset'])
                            if row['SUCCESS'] == 'yes':
                                if row['dataset'] not in datasets_success:
                                    datasets_success.append(row['dataset'])
                            if row['SUCCESS'] == 'no':
                                unsuccessfull = True
                                unsuccessfull_all = True


            # enlever les doublons de la liste
            # Set est une structure de données non ordonnée qui ne contient que des éléments uniques
            #rows_all_files = list(set(rows_all_files))  # ne marche pas car liste de listes
            rows_all_files_nodouble = []
            for l in rows_all_files:
                if l not in rows_all_files_nodouble:
                    rows_all_files_nodouble.append(l)

            # trier par dataset, ...
            # commencer par dernier à trier et finir par l'attribut majeur afin que l'ordre soit correct après plusieurs tris
            # nécessaire car certains sont en reverse
            rows_all_files_nodouble.sort(key=lambda row: row['Rwork'] or "", reverse=False)
            rows_all_files_nodouble.sort(key=lambda row: row['Nb of ligands found'] or "", reverse=True)
            rows_all_files_nodouble.sort(key=lambda row: row['Completeness'] or "", reverse=True)
            rows_all_files_nodouble.sort(key=lambda row: row['Ligand'] or "") # or "": gérer cellules avec None Type
            # TypeError: '<' not supported between instances of 'NoneType' and 'NoneType'
            #rows_all_files_nodouble.sort(key=lambda row: row['mode'], reverse=True)
            rows_all_files_nodouble.sort(key=lambda row: row['ligand search'], reverse=True)
            rows_all_files_nodouble.sort(key=lambda row: row['SUCCESS'], reverse=True)
            rows_all_files_nodouble.sort(key=lambda row: row['dataset'].lower())

            # copier pour le summary général
            if len(res_dirs) >1:
                # ajouter le nom du dossier RESULTS en première colonne
                # opérateur de décompression du dictionnaire (**)
                for row in rows_all_files_nodouble:
                    first_col = {'RESULTS dir': Path(dir).name.split('RESULTS_')[1]}
                    new_row = OrderedDict({**first_col, **row})
                    rows_summary_files.append(new_row)



            #verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), summary_log)

            # Copier la liste dans le fichier summary
            summary = phenix_dir.joinpath(dir, 'SUMMARY_' + dir + '.csv')
            verbose.print_and_write("Summary file created: " + summary.name + " in " + dir + " folder", summary_log)
            with open(summary, 'w', newline='') as csvfile_sum:
                csv_writer = csv.writer(csvfile_sum, quoting=csv.QUOTE_MINIMAL)
                csv_writer.writerow(header)
                for row in rows_all_files_nodouble:
                    csv_writer.writerow(row.values())
            # copier la liste dans le fichier summary only success // option possible qu'en lançant la sous-commande summary
            if args.subcommands == 'summary' and args.only_success and unsuccessfull:
                summary_success = phenix_dir.joinpath(dir, 'SUMMARY_' + dir + '_onlySUCCESS.csv')
                verbose.print_and_write("Summary file with only successfull treatments created: " + summary_success.name + " in " + dir + " folder", summary_log)
                with open(summary_success, 'w', newline='') as csvfile_sum_success:
                    csv_writer = csv.writer(csvfile_sum_success, quoting=csv.QUOTE_MINIMAL)
                    csv_writer.writerow(header)
                    for row in rows_all_files_nodouble:
                        if row['SUCCESS'] == 'yes':
                            csv_writer.writerow(row.values())
        else:
            verbose.print_and_write("No RESULT csv files ...\n", summary_log)

    """Make a summary of summary files if several RESULTS directories"""
    # générer le summary des summary
    if len(res_dirs) > 1:
        # concatener les entrées des fichiers summary
        # en ajoutant une première colonne avec l'info de quel RESULTS cela provient
        header_all = ['RESULTS dir']
        header_all.extend(header)

    if len(res_dirs) >1:
        verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                   "General summary file "), summary_log)

        # tri comme précédemment
        rows_summary_files.sort(key=lambda row: row['Rwork'] or "", reverse=False)
        rows_summary_files.sort(key=lambda row: row['Nb of ligands found'] or "", reverse=True)
        rows_summary_files.sort(key=lambda row: row['Completeness'] or "", reverse=True)
        rows_summary_files.sort(key=lambda row: row['Ligand'] or "")  # or "": gérer cellules avec None Type
        # TypeError: '<' not supported between instances of 'NoneType' and 'NoneType'
        #rows_summary_files.sort(key=lambda row: row['mode'], reverse=True)
        rows_summary_files.sort(key=lambda row: row['ligand search'], reverse=True)
        rows_summary_files.sort(key=lambda row: row['SUCCESS'], reverse=True)
        rows_summary_files.sort(key=lambda row: row['dataset'].lower())

        summary_all = phenix_dir.joinpath('SUMMARY_RESULTS_ALL.csv')
        verbose.print_and_write(
            "General summary file created: " + summary_all.name + " in " + summary_all.parents[
                0].name + " folder", summary_log)
        with open(summary_all, 'w', newline='') as csvfile_sumall:
            csv_writer = csv.writer(csvfile_sumall, quoting=csv.QUOTE_MINIMAL)
            csv_writer.writerow(header_all)
            for row in rows_summary_files:
                csv_writer.writerow(row.values())

        # summary des summary avec seulement les success
        if args.only_success and unsuccessfull_all:
            summary_all_success = phenix_dir.joinpath('SUMMARY_RESULTS_ALL_onlySUCCESS.csv')
            verbose.print_and_write(
                "General summary file, with only successful treatments created: " + summary_all_success.name + " in " +
                summary_all_success.parents[
                    0].name + " folder, in case of unsuccessfull treatments", summary_log)
            with open(summary_all_success, 'w', newline='') as csvfile_sumall_success:
                csv_writer = csv.writer(csvfile_sumall_success, quoting=csv.QUOTE_MINIMAL)
                csv_writer.writerow(header_all)
                for row in rows_summary_files:
                    if row['SUCCESS'] == 'yes':
                        csv_writer.writerow(row.values())

    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                               "Final information"), summary_log)
    # datasets not treated :
    datasets_nottreated = sorted(list(set(datasets_all) - set(datasets_treated)))
    if datasets_nottreated:
        verbose.print_and_write("Datasets not treated:", summary_log)
        for dataset in datasets_nottreated:
            verbose.print_and_write(dataset, summary_log)
    else:
        verbose.print_and_write("All datasets were treated ...", summary_log)
    # datasets with no success :
    datasets_nosuccess = sorted(list(set(datasets_treated) - set(datasets_success)))
    if datasets_nosuccess:
        verbose.print_and_write("Datasets without any successfull treatment:", summary_log)
        for dataset in datasets_nosuccess:
            verbose.print_and_write(dataset, summary_log)
    else:
        verbose.print_and_write("No datasets without any successfull treatment ...", summary_log)
    verbose.print_and_write(verbose.chaine_xfois(config.motif, config.nb_motifs), summary_log)
    verbose.print_and_write("\nDone ...", summary_log)
    os.chdir(work_dir)
    return summary_log


